# Kotlin Quiz App
Kotlin Quiz is an engaging Android application that allows users to test their knowledge in Kotilin Programming language through fun and interactive quizzes. This app is built with a clean code architecture following the MVVM pattern, ensuring maintainability and scalability. The app comprises three main layers: App (Presentation), Domain, and Data, each with its unique set of responsibilities.

## Screenshots
<p align="center" padding = "40" >
    <img src="https://github.com/NiranjanNlc/Kotilin-Quiz/assets/25600880/b77fea82-e8ef-42c5-b318-177e1149474f.png" alt="Screenshot 4" width="200"/>
    <img src="https://github.com/NiranjanNlc/Kotilin-Quiz/assets/25600880/b27a2053-a999-433e-89e7-d65247b876f5.png" alt="Screenshot 3" width="200"/>
  <img src="https://github.com/NiranjanNlc/Kotilin-Quiz/assets/25600880/c5ba0742-fe8f-4ef7-bdbf-92934d2ca6a2.png" alt="Screenshot 2" width="200"/>
  <img src="https://github.com/NiranjanNlc/Kotilin-Quiz/assets/25600880/6ce64d5a-5e32-4a19-8fd9-d1666bbca4fb.png" alt="Screenshot 1" width="200"/>
</p>

## Features

- Interactive Quizzes: Test your knowledge with multiple-choice quizzes on diverse topics.
- MVVM Architecture: Follows the Model-View-ViewModel design pattern for clear separation of concerns.
- Clean Code: Well-organized codebase following clean code principles for maintainability.
- Unit Testing: Extensive unit tests for the Domain Layer using test-driven development (TDD).
- Integration Testing: Data Layer undergoes integration tests for smooth interaction with data sources.
- Dependency Injection: Utilizes Dagger Hilt for easy and efficient dependency injection.
- Navigation Component: Simplifies navigation between fragments and ensures a smooth user experience.
- Android Architecture Components: Employs LiveData and ViewModel for seamless UI updates.
- Room Database: Store quiz data in the app's local database for offline access.



## Setup Instructions

1. Clone the repository to your local machine using `git clone https://github.com/your_username/kotlin-quiz-app.git`.
2. Open the project in Android Studio.
3. Build and run the app on your Android device or emulator.

## Dependencies!

- Kotlin: 1.5.30
- Android Gradle Plugin: 8.0.1
- Kotlin Gradle Plugin: 1.9.0
- Dagger Hilt: 2.47
- AndroidX Libraries: core-ktx, appcompat, material, constraintlayout, lifecycle-livedata-ktx, lifecycle-viewmodel-ktx, legacy-support-v4, fragment
- Room Database: room-ktx, room-runtime
- Navigation Component: navigation-fragment-ktx, navigation-ui-ktx
- Testing: junit, junit-jupiter-api, org.mockito.kotlin:mockito-kotlin

## Contributing

Contributions to the Kotlin Quiz App are always welcome! If you find any issues or have suggestions for improvements, please feel free to open an issue or submit a pull request.

## License

This project is licensed under the MIT License 

---

We hope you enjoy using Kotlin Quiz and find it both educational and entertaining. If you have any questions or feedback, please don't hesitate to reach out to us.

Happy quizzing! 🚀
